"use strict";

exports.__esModule = true;
exports["default"] = extractOpenArguments;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _astToReference = require("./ast/to-reference");

var _astToReference2 = _interopRequireDefault(_astToReference);

var _astIsLiteralOrUndefined = require("./ast/is-literal-or-undefined");

var _astIsLiteralOrUndefined2 = _interopRequireDefault(_astIsLiteralOrUndefined);

var _hoistStatics = require("./hoist-statics");

var _hoistStatics2 = _interopRequireDefault(_hoistStatics);

// Extracts attributes into the appropriate
// attribute array. Static attributes and the key
// are placed into static attributes, and expressions
// are placed into the variadic attributes.

function extractOpenArguments(t, scope, file, attributes, _ref) {
  var eager = _ref.eager;
  var hoist = _ref.hoist;

  var attributeDeclarators = [];
  var attrs = [];
  var hasSpread = false;
  var key = null;
  var statics = [];
  var keyIndex = -1;
  var staticAssignment = null;

  attributes.forEach(function (attribute, i) {
    if (t.isJSXSpreadAttribute(attribute)) {
      hasSpread = true;
      attrs.push(attribute);
      return;
    }

    var attr = _astToReference2["default"](t, attribute.name);
    var name = attr.value;
    var value = attribute.value;

    if (t.isJSXExpressionContainer(value)) {
      value = value.expression;

      if (eager && !_astIsLiteralOrUndefined2["default"](t, value) && !value._iDOMwasJSX) {
        var ref = scope.generateUidIdentifierBasedOnNode(value);
        attributeDeclarators.push(t.variableDeclarator(ref, value));
        value = ref;
      }
    } else if (!value) {
      value = t.literal(true);
    }

    var literal = _astIsLiteralOrUndefined2["default"](t, value);

    if (name === "key") {
      key = value;
      if (hoist && !(literal || eager)) {
        value = t.literal("");
        keyIndex = (i << 1) + 1;
      }

      if (hoist && eager && !literal) {
        attrs.push(attr, value);
      } else {
        statics.push(attr, value);
      }

      return;
    }

    if (literal) {
      statics.push(attr, value);
    } else {
      attrs.push(attr, value);
    }
  });

  if (!attrs.length) {
    attrs = null;
  }
  if (statics.length) {
    statics = t.arrayExpression(statics);
    if (hoist) {
      var _hoist = _hoistStatics2["default"](t, scope, file, statics, key, keyIndex);
      statics = _hoist.id;
      staticAssignment = _hoist.staticAssignment;
    }
  } else {
    statics = null;
  }

  return { key: key, keyIndex: keyIndex, statics: statics, attrs: attrs, attributeDeclarators: attributeDeclarators, staticAssignment: staticAssignment, hasSpread: hasSpread };
}

module.exports = exports["default"];