"use strict";

exports.__esModule = true;
exports.setupInjector = setupInjector;
exports["default"] = inject;
var namespace = "incremental-dom-helpers";

function getHelperRef(file, helper) {
  return file.get(namespace)[helper];
}

function setHelperRef(file, helper, value) {
  return file.get(namespace)[helper] = value;
}

// Sets up the needed data maps for injecting runtime helpers.

function setupInjector(program, parent, scope, file) {
  // A map to store helper variable references
  // for each file
  file.setDynamic(namespace, function () {
    return Object.create(null);
  });
}

// Injects a helper function defined by helperAstFn into the current file at
// the top scope.

function inject(t, file, helper, helperAstFn) {
  var dependencyInjectors = arguments.length <= 4 || arguments[4] === undefined ? {} : arguments[4];

  var helperRef = getHelperRef(file, helper);
  if (helperRef) {
    return helperRef;
  }

  helperRef = file.scope.generateUidIdentifier(helper);
  setHelperRef(file, helper, helperRef);

  var dependencyRefs = {};

  for (var dependency in dependencyInjectors) {
    var dependencyRef = getHelperRef(file, dependency);

    if (!dependencyRef) {
      dependencyRef = dependencyInjectors[dependency](t, file);
    }

    dependencyRefs[dependency] = dependencyRef;
  }

  file.path.unshiftContainer("body", helperAstFn(t, file, helperRef, dependencyRefs));

  return helperRef;
}