"use strict";

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _helpersAstToFunctionCall = require("./helpers/ast/to-function-call");

var _helpersAstToFunctionCall2 = _interopRequireDefault(_helpersAstToFunctionCall);

var _helpersAstToReference = require("./helpers/ast/to-reference");

var _helpersAstToReference2 = _interopRequireDefault(_helpersAstToReference);

var _helpersGetOption = require("./helpers/get-option");

var _helpersGetOption2 = _interopRequireDefault(_helpersGetOption);

var _helpersIdomMethod = require("./helpers/idom-method");

var _helpersIdomMethod2 = _interopRequireDefault(_helpersIdomMethod);

var _helpersAttributesToAttrCalls = require("./helpers/attributes-to-attr-calls");

var _helpersAttributesToAttrCalls2 = _interopRequireDefault(_helpersAttributesToAttrCalls);

var _helpersBuildChildren = require("./helpers/build-children");

var _helpersBuildChildren2 = _interopRequireDefault(_helpersBuildChildren);

var _helpersExtractOpenArguments = require("./helpers/extract-open-arguments");

var _helpersExtractOpenArguments2 = _interopRequireDefault(_helpersExtractOpenArguments);

var _helpersFindOtherJsx = require("./helpers/find-other-jsx");

var _helpersFindOtherJsx2 = _interopRequireDefault(_helpersFindOtherJsx);

var _helpersFlattenExpressions = require("./helpers/flatten-expressions");

var _helpersFlattenExpressions2 = _interopRequireDefault(_helpersFlattenExpressions);

var _helpersStatementsWithReturnLast = require("./helpers/statements-with-return-last");

var _helpersStatementsWithReturnLast2 = _interopRequireDefault(_helpersStatementsWithReturnLast);

var _helpersReplaceArrow = require("./helpers/replace-arrow");

var _helpersReplaceArrow2 = _interopRequireDefault(_helpersReplaceArrow);

var _helpersHoistStatics = require("./helpers/hoist-statics");

var _helpersEagerlyDeclare = require("./helpers/eagerly-declare");

var _helpersEagerlyDeclare2 = _interopRequireDefault(_helpersEagerlyDeclare);

var _helpersInject = require("./helpers/inject");

var _helpersRuntimeJsxWrapper = require("./helpers/runtime/jsx-wrapper");

var _helpersRuntimeJsxWrapper2 = _interopRequireDefault(_helpersRuntimeJsxWrapper);

exports["default"] = function (_ref) {
  var Plugin = _ref.Plugin;
  var t = _ref.types;

  return new Plugin("incremental-dom", { visitor: {
      Program: {
        enter: [_helpersInject.setupInjector, _helpersHoistStatics.setupHoists],

        exit: function exit(program, parent, scope, file) {
          _helpersHoistStatics.hoistStatics(t, file, this);
        }
      },

      JSXElement: {
        enter: function enter(node) {
          var inAssignment = false;
          var inAttribute = false;
          var inCallExpression = false;
          var inCollection = false;
          var inReturnStatement = false;
          var containingJSXElement = undefined;
          var last = node;

          this.findParent(function (path) {
            if (path.isJSXElement()) {
              containingJSXElement = path;
              return true;
            }
            if (path.isArrowFunctionExpression()) {
              inReturnStatement = inReturnStatement || !t.isBlockStatement(path.node.body);
              return true;
            }
            if (path.isFunction() || path.isProgram()) {
              return true;
            }
            if (path.isSequenceExpression()) {
              var expressions = path.node.expressions;
              var index = expressions.indexOf(last);
              if (index !== expressions.length - 1) {
                return true;
              }
            }
            if (path.isJSXAttribute()) {
              inAttribute = true;
            } else if (path.isAssignmentExpression() || path.isVariableDeclarator()) {
              inAssignment = true;
            } else if (path.isArrayExpression() || path.isObjectExpression()) {
              inCollection = true;
            } else if (path.isReturnStatement()) {
              inReturnStatement = true;
            } else if (path.isCallExpression()) {
              inCallExpression = true;
            }
            last = path.node;
          });

          // Values are useless if they aren't assigned.
          // ```
          //   var a = 1;
          //   <div /> // Useless JSX node
          // ```
          if (!(inReturnStatement || inAssignment || inCallExpression || containingJSXElement)) {
            throw this.errorWithNode("Unused JSX Elements aren't supported.");
          }

          var containerNeedsWrapper = containingJSXElement ? containingJSXElement.getData("containerNeedsWrapper") || containingJSXElement.getData("needsWrapper") : false;
          var needsWrapper = inAttribute || inAssignment || inCollection || inCallExpression;
          if (!containingJSXElement && !needsWrapper) {
            // Determine if there are JSXElements in a higher scope.
            needsWrapper = _helpersFindOtherJsx2["default"](this);
          }

          // Tie a child JSXElement's eager declarations with the parent's, so
          // so all declarations come before the element.
          var eagerDeclarators = containingJSXElement ? containingJSXElement.getData("eagerDeclarators") : [];
          var staticAssignments = containingJSXElement ? containingJSXElement.getData("staticAssignments") : [];

          this.setData("containerNeedsWrapper", containerNeedsWrapper);
          this.setData("containingJSXElement", containingJSXElement);
          this.setData("eagerDeclarators", eagerDeclarators);
          this.setData("needsWrapper", needsWrapper);
          this.setData("staticAssignments", staticAssignments);
        },

        exit: function exit(node, parent, scope, file) {
          var _data = this.data;
          var containerNeedsWrapper = _data.containerNeedsWrapper;
          var containingJSXElement = _data.containingJSXElement;
          var eagerDeclarators = _data.eagerDeclarators;
          var staticAssignments = _data.staticAssignments;
          var needsWrapper = _data.needsWrapper;

          var eager = needsWrapper || containerNeedsWrapper;
          var explicitReturn = t.isReturnStatement(parent);
          var implicitReturn = t.isArrowFunctionExpression(parent);

          // Filter out empty children, and transform JSX expressions
          // into normal expressions.
          var openingElement = node.openingElement;
          var closingElement = node.closingElement;

          var _buildChildren = _helpersBuildChildren2["default"](t, scope, file, node.children, { eager: eager });

          var children = _buildChildren.children;
          var eagerChildren = _buildChildren.eagerChildren;

          eagerDeclarators.push.apply(eagerDeclarators, eagerChildren);

          var elements = [openingElement].concat(children);
          if (closingElement) {
            elements.push(closingElement);
          }

          // If we're inside a JSX node, flattening expressions
          // may force us into an unwanted function scope.
          if (t.isJSXElement(parent)) {
            return elements;
          }

          // Expressions Containers must contain an expression and not statements.
          // This will be flattened out into statements later.
          if (containingJSXElement && !needsWrapper) {
            var sequence = t.sequenceExpression(elements);
            // Mark this sequence as a JSX Element so we can avoid an unnecessary
            // renderArbitrary call.
            sequence._iDOMwasJSX = true;
            return sequence;
          }

          if (explicitReturn || implicitReturn || needsWrapper) {
            // Transform (recursively) any sequence expressions into a series of
            // statements.
            elements = _helpersFlattenExpressions2["default"](t, elements);

            // Ensure the last statement returns the DOM element.
            elements = _helpersStatementsWithReturnLast2["default"](t, elements);
          }

          if (!containingJSXElement && eagerDeclarators.length) {
            _helpersEagerlyDeclare2["default"](t, scope, this, eagerDeclarators);
          }

          if (!containingJSXElement && staticAssignments.length) {
            elements = [].concat(staticAssignments, elements);
          }

          if (needsWrapper) {
            // Create a wrapper around our element, and mark it as a one so later
            // child expressions can identify and "render" it.
            var jsxWrapperRef = _helpersRuntimeJsxWrapper2["default"](t, file);
            var wrapper = _helpersAstToFunctionCall2["default"](t, jsxWrapperRef, [t.functionExpression(null, [], t.blockStatement(elements))]);
            wrapper._iDOMwasJSX = true;
            return wrapper;
          }

          openingElement._iDOMisRoot = true;

          // This is the main JSX element. Replace the return statement
          // with all the nested calls, returning the main JSX element.
          if (implicitReturn) {
            _helpersReplaceArrow2["default"](t, this.parentPath, elements);
          } else if (explicitReturn) {
            this.parentPath.replaceWithMultiple(elements);
          } else {
            return elements;
          }
        }
      },

      JSXOpeningElement: {
        exit: function exit(node, parent, scope, file) {
          var tag = _helpersAstToReference2["default"](t, node.name);

          var JSXElement = this.parentPath;
          // Only eagerly evaluate our attributes if we will be wrapping the element.
          var eager = JSXElement.getData("needsWrapper") || JSXElement.getData("containerNeedsWrapper");
          var eagerDeclarators = JSXElement.getData("eagerDeclarators");
          var hoist = _helpersGetOption2["default"](file, "hoist");
          var staticAssignments = JSXElement.getData("staticAssignments");

          var _extractOpenArguments = _helpersExtractOpenArguments2["default"](t, scope, file, node.attributes, { eager: eager, hoist: hoist });

          var key = _extractOpenArguments.key;
          var statics = _extractOpenArguments.statics;
          var attrs = _extractOpenArguments.attrs;
          var attributeDeclarators = _extractOpenArguments.attributeDeclarators;
          var staticAssignment = _extractOpenArguments.staticAssignment;
          var hasSpread = _extractOpenArguments.hasSpread;

          // Push any eager attribute declarators onto the element's list of
          // eager declarations.
          eagerDeclarators.push.apply(eagerDeclarators, attributeDeclarators);
          if (staticAssignment) {
            staticAssignments.push(staticAssignment);
          }

          // Only push arguments if they're needed
          var args = [tag];
          if (key || statics) {
            args.push(key || t.literal(null));
          }
          if (statics) {
            args.push(statics);
          }

          // If there is a spread element, we need to use
          // the elementOpenStart/elementOpenEnd syntax.
          // This allows spreads to be transformed into
          // attr(name, value) calls.
          if (hasSpread) {
            var attrCalls = _helpersAttributesToAttrCalls2["default"](t, file, attrs);

            var expressions = [_helpersAstToFunctionCall2["default"](t, _helpersIdomMethod2["default"](file, "elementOpenStart"), args)].concat(attrCalls, [_helpersAstToFunctionCall2["default"](t, _helpersIdomMethod2["default"](file, "elementOpenEnd"), [tag])]);
            if (node.selfClosing) {
              expressions.push(_helpersAstToFunctionCall2["default"](t, _helpersIdomMethod2["default"](file, "elementClose"), [tag]));
            }

            return t.sequenceExpression(expressions);
          }

          if (attrs) {
            // Only push key and statics if they have not
            // already been pushed.
            if (!statics) {
              if (!key) {
                args.push(t.literal(null));
              }
              args.push(t.literal(null));
            }

            args.push.apply(args, attrs);
          }

          var elementFunction = node.selfClosing ? "elementVoid" : "elementOpen";
          return _helpersAstToFunctionCall2["default"](t, _helpersIdomMethod2["default"](file, elementFunction), args);
        }
      },

      JSXClosingElement: {
        exit: function exit(node, parent, scope, file) {
          return _helpersAstToFunctionCall2["default"](t, _helpersIdomMethod2["default"](file, "elementClose"), [_helpersAstToReference2["default"](t, node.name)]);
        }
      },

      JSXNamespacedName: function JSXNamespacedName() {
        throw this.errorWithNode("JSX Namespaces aren't supported.");
      }
    } });
};

module.exports = exports["default"];